@@ -2,18 +2,17 @@
 # this repository contains the full copyright notices and license terms.
 import os
 import time
-import logging
+import warnings
 from email import charset
 
-__version__ = "4.2.1"
-logger = logging.getLogger(__name__)
+__version__ = "4.4.1"
 
 os.environ['TZ'] = 'UTC'
 if hasattr(time, 'tzset'):
     time.tzset()
 
 if time.tzname[0] != 'UTC':
-    logger.error('Timezone must be set to UTC instead of %s', time.tzname[0])
+    warnings.warn('Timezone must be set to UTC instead of %s', time.tzname[0])
 
 # set email encoding for utf-8 to 'quoted-printable'
 charset.add_charset('utf-8', charset.QP, charset.QP)
