@@ -6,14 +6,16 @@ import re
 import os
 import urllib
 from decimal import Decimal
+from threading import RLock
 
 try:
     from psycopg2cffi import compat
     compat.register()
 except ImportError:
     pass
-from psycopg2 import connect
-from psycopg2.pool import ThreadedConnectionPool
+from psycopg2 import connect, Binary
+from psycopg2.pool import ThreadedConnectionPool, PoolError
+from psycopg2.extensions import cursor
 from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
 from psycopg2.extensions import register_type, register_adapter
@@ -27,7 +29,7 @@ from psycopg2 import OperationalError as DatabaseOperationalError
 
 from sql import Flavor
 
-from trytond.backend.database import DatabaseInterface
+from trytond.backend.database import DatabaseInterface, SQLType
 from trytond.config import config, parse_uri
 
 __all__ = ['Database', 'DatabaseIntegrityError', 'DatabaseOperationalError']
@@ -51,30 +53,54 @@ def replace_special_values(s, **mapping):
     return s
 
 
+class LoggingCursor(cursor):
+    def execute(self, sql, args=None):
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug(self.mogrify(sql, args))
+        cursor.execute(self, sql, args)
+
+
 class Database(DatabaseInterface):
 
+    _lock = RLock()
     _databases = {}
     _connpool = None
     _list_cache = None
     _list_cache_timestamp = None
     _version_cache = {}
+    _search_path = None
+    _current_user = None
+    _has_returning = None
     flavor = Flavor(ilike=True)
 
-    def __new__(cls, name='template1'):
-        if name in cls._databases:
-            return cls._databases[name]
-        return DatabaseInterface.__new__(cls, name=name)
+    TYPES_MAPPING = {
+        'INTEGER': SQLType('INT4', 'INT4'),
+        'BIGINT': SQLType('INT8', 'INT8'),
+        'FLOAT': SQLType('FLOAT8', 'FLOAT8'),
+        'BLOB': SQLType('BYTEA', 'BYTEA'),
+        'DATETIME': SQLType('TIMESTAMP', 'TIMESTAMP(0)'),
+        'TIMESTAMP': SQLType('TIMESTAMP', 'TIMESTAMP(6)'),
+        }
 
-    def __init__(self, name='template1'):
-        super(Database, self).__init__(name=name)
-        self._databases.setdefault(name, self)
-        self._search_path = None
-        self._current_user = None
+    def __new__(cls, name='template1'):
+        with cls._lock:
+            if name in cls._databases:
+                return cls._databases[name]
+            inst = DatabaseInterface.__new__(cls, name=name)
+            cls._databases[name] = inst
+
+            logger.info('connect to "%s"', name)
+            minconn = config.getint('database', 'minconn', default=1)
+            maxconn = config.getint('database', 'maxconn', default=64)
+            inst._connpool = ThreadedConnectionPool(
+                minconn, maxconn, cls.dsn(name),
+                cursor_factory=LoggingCursor)
+
+            return inst
 
     @classmethod
     def dsn(cls, name):
         uri = parse_uri(config.get('database', 'uri'))
-        assert uri.scheme == 'postgresql'
         host = uri.hostname and "host=%s" % uri.hostname or ''
         port = uri.port and "port=%s" % uri.port or ''
         name = "dbname=%s" % name
@@ -84,19 +110,19 @@ class Database(DatabaseInterface):
         return '%s %s %s %s %s' % (host, port, name, user, password)
 
     def connect(self):
-        if self._connpool is not None:
-            return self
-        logger.info('connect to "%s"', self.name)
-        minconn = config.getint('database', 'minconn', default=1)
-        maxconn = config.getint('database', 'maxconn', default=64)
-        self._connpool = ThreadedConnectionPool(
-            minconn, maxconn, self.dsn(self.name))
         return self
 
     def get_connection(self, autocommit=False, readonly=False):
-        if self._connpool is None:
-            self.connect()
-        conn = self._connpool.getconn()
+        for count in range(config.getint('database', 'retry'), -1, -1):
+            try:
+                conn = self._connpool.getconn()
+                break
+            except PoolError:
+                if count and not self._connpool.closed:
+                    logger.info('waiting a connection')
+                    time.sleep(1)
+                    continue
+                raise
         if autocommit:
             conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
         else:
@@ -110,10 +136,9 @@ class Database(DatabaseInterface):
         self._connpool.putconn(connection, close=close)
 
     def close(self):
-        if self._connpool is None:
-            return
-        self._connpool.closeall()
-        self._connpool = None
+        with self._lock:
+            self._connpool.closeall()
+            self._databases.pop(self.name)
 
     @classmethod
     def create(cls, connection, database_name):
@@ -126,7 +151,7 @@ class Database(DatabaseInterface):
     def drop(self, connection, database_name):
         cursor = connection.cursor()
         cursor.execute('DROP DATABASE "' + database_name + '"')
-        Database._list_cache = None
+        self.__class__._list_cache = None
 
     def get_version(self, connection):
         if self.name not in self._version_cache:
@@ -137,89 +162,11 @@ class Database(DatabaseInterface):
                 RE_VERSION.search(version).groups()))
         return self._version_cache[self.name]
 
-    @staticmethod
-    def dump(database_name):
-        from trytond.tools import exec_command_pipe
-
-        cmd = ['pg_dump', '--format=c', '--no-owner']
-        env = {}
-        uri = parse_uri(config.get('database', 'uri'))
-        if uri.username:
-            cmd.append('--username=' + uri.username)
-        if uri.hostname:
-            cmd.append('--host=' + uri.hostname)
-        if uri.port:
-            cmd.append('--port=' + str(uri.port))
-        if uri.password:
-            # if db_password is set in configuration we should pass
-            # an environment variable PGPASSWORD to our subprocess
-            # see libpg documentation
-            env['PGPASSWORD'] = uri.password
-        cmd.append(database_name)
-
-        pipe = exec_command_pipe(*tuple(cmd), env=env)
-        pipe.stdin.close()
-        data = pipe.stdout.read()
-        res = pipe.wait()
-        if res:
-            raise Exception('Couldn\'t dump database!')
-        return data
-
-    @staticmethod
-    def restore(database_name, data):
-        from trytond.tools import exec_command_pipe
-
-        database = Database().connect()
-        connection = database.get_connection(autocommit=True)
-        database.create(connection, database_name)
-        database.close()
-
-        cmd = ['pg_restore', '--no-owner']
-        env = {}
-        uri = parse_uri(config.get('database', 'uri'))
-        if uri.username:
-            cmd.append('--username=' + uri.username)
-        if uri.hostname:
-            cmd.append('--host=' + uri.hostname)
-        if uri.port:
-            cmd.append('--port=' + str(uri.port))
-        if uri.password:
-            env['PGPASSWORD'] = uri.password
-        cmd.append('--dbname=' + database_name)
-        args2 = tuple(cmd)
-
-        if os.name == "nt":
-            tmpfile = (os.environ['TMP'] or 'C:\\') + os.tmpnam()
-            with open(tmpfile, 'wb') as fp:
-                fp.write(data)
-            args2 = list(args2)
-            args2.append(' ' + tmpfile)
-            args2 = tuple(args2)
-
-        pipe = exec_command_pipe(*args2, env=env)
-        if not os.name == "nt":
-            pipe.stdin.write(data)
-        pipe.stdin.close()
-        res = pipe.wait()
-        if res:
-            raise Exception('Couldn\'t restore database')
-
-        database = Database(database_name).connect()
-        cursor = database.get_connection().cursor()
-        if not database.test():
-            cursor.close()
-            database.close()
-            raise Exception('Couldn\'t restore database!')
-        cursor.close()
-        database.close()
-        Database._list_cache = None
-        return True
-
     def list(self):
         now = time.time()
         timeout = config.getint('session', 'timeout')
-        res = Database._list_cache
-        if res and abs(Database._list_cache_timestamp - now) < timeout:
+        res = self.__class__._list_cache
+        if res and abs(self.__class__._list_cache_timestamp - now) < timeout:
             return res
 
         connection = self.get_connection()
@@ -236,8 +183,8 @@ class Database(DatabaseInterface):
                 continue
         self.put_connection(connection)
 
-        Database._list_cache = res
-        Database._list_cache_timestamp = now
+        self.__class__._list_cache = res
+        self.__class__._list_cache_timestamp = now
         return res
 
     def init(self):
@@ -252,9 +199,9 @@ class Database(DatabaseInterface):
                     cursor.execute(line)
 
         for module in ('ir', 'res'):
-            state = 'uninstalled'
+            state = 'not activated'
             if module in ('ir', 'res'):
-                state = 'to install'
+                state = 'to activate'
             info = get_module_info(module)
             cursor.execute('SELECT NEXTVAL(\'ir_module_id_seq\')')
             module_id = cursor.fetchone()[0]
@@ -353,6 +300,35 @@ class Database(DatabaseInterface):
                 self.put_connection(connection)
         return self._search_path
 
+    def has_returning(self):
+        if self._has_returning is None:
+            connection = self.get_connection()
+            try:
+                # RETURNING clause is available since PostgreSQL 8.2
+                self._has_returning = self.get_version(connection) >= (8, 2)
+            finally:
+                self.put_connection(connection)
+        return self._has_returning
+
+    def has_select_for(self):
+        return True
+
+    def has_window_functions(self):
+        return True
+
+    def sql_type(self, type_):
+        if type_ in self.TYPES_MAPPING:
+            return self.TYPES_MAPPING[type_]
+        if type_.startswith('VARCHAR'):
+            return SQLType('VARCHAR', type_)
+        return SQLType(type_, type_)
+
+    def sql_format(self, type_, value):
+        if type_ == 'BLOB':
+            if value is not None:
+                return Binary(value)
+        return value
+
 register_type(UNICODE)
 if PYDATE:
     register_type(PYDATE)
