@@ -6,12 +6,12 @@ Miscelleanous tools used by tryton
 """
 import os
 import sys
-import subprocess
 from array import array
 from itertools import islice
 import types
 import io
 import warnings
+import importlib
 
 from sql import Literal
 from sql.operators import Or
@@ -19,35 +19,6 @@ from sql.operators import Or
 from trytond.const import OPERATORS
 
 
-def find_in_path(name):
-    if os.name == "nt":
-        sep = ';'
-    else:
-        sep = ':'
-    path = [directory for directory in os.environ['PATH'].split(sep)
-            if os.path.isdir(directory)]
-    for directory in path:
-        val = os.path.join(directory, name)
-        if os.path.isfile(val) or os.path.islink(val):
-            return val
-    return name
-
-
-def exec_command_pipe(name, *args, **kwargs):
-    prog = find_in_path(name)
-    if not prog:
-        raise Exception('Couldn\'t find %s' % name)
-    if os.name == "nt":
-        cmd = '"' + prog + '" ' + ' '.join(args)
-    else:
-        cmd = prog + ' ' + ' '.join(args)
-    child_env = dict(os.environ)
-    if kwargs.get('env'):
-        child_env.update(kwargs['env'])
-    return subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE, env=child_env)
-
-
 def file_open(name, mode="r", subdir='modules', encoding=None):
     """Open a file from the root dir, using a subdir folder."""
     from trytond.modules import EGG_MODULES
@@ -61,7 +32,7 @@ def file_open(name, mode="r", subdir='modules', encoding=None):
         "Join paths and ensure it still below root"
         path = os.path.join(root, *paths)
         path = os.path.normpath(path)
-        if not path.startswith(root):
+        if not path.startswith(os.path.join(root, '')):
             raise IOError("Permission denied: %s" % name)
         return path
 
@@ -105,6 +76,12 @@ def file_open(name, mode="r", subdir='modules', encoding=None):
     raise IOError('File not found : %s ' % name)
 
 
+def get_parent_language(code):
+    for sep in ['@', '_']:
+        if sep in code:
+            return code.rsplit(sep, 1)[0]
+
+
 def get_smtp_server():
     """
     Instanciate, configure and return a SMTP or SMTP_SSL instance from
@@ -182,23 +159,6 @@ def memoize(maxsize):
     return wrap
 
 
-def mod10r(number):
-    """
-    Recursive mod10
-
-    :param number: a number
-    :return: the same number completed with the recursive modulo base 10
-    """
-    codec = [0, 9, 4, 6, 8, 2, 7, 1, 3, 5]
-    report = 0
-    result = ""
-    for digit in number:
-        result += digit
-        if digit.isdigit():
-            report = codec[(int(digit) + report) % 10]
-    return result + str((10 - report) % 10)
-
-
 def reduce_ids(field, ids):
     '''
     Return a small SQL expression for the list of ids and the sql column
@@ -282,3 +242,17 @@ def is_instance_method(cls, method):
         type_ = klass.__dict__.get(method)
         if type_ is not None:
             return isinstance(type_, types.FunctionType)
+
+
+def resolve(name):
+    "Resolve a dotted name to a global object."
+    name = name.split('.')
+    used = name.pop(0)
+    found = importlib.import_module(used)
+    for n in name:
+        used = used + '.' + n
+        try:
+            found = getattr(found, n)
+        except AttributeError:
+            found = importlib.import_module(used)
+    return found
